<?php
require('./tools/formtools.php');
require('./tools/seqtools.php');// include sequence functions, parsing etc.

// Form buttons from main form
$submit = $_POST['submit'];
$submit_example = $_POST['submit_example'];

function parse_sequences ($inputseqs) {
  $sequences = parse_fasta($inputseqs, 0, "-"); // returns array of sequences, each consist of two elements: header and sequence, i.e. $var[0][0] = header of seq 1, $var[0][1] = sequence 1 etc...
  if (count($sequences) ==0){
    $error = "Couldn't parse FASTA input correctly, please submit fasta formatted sequences";
    return array(1,$error);
  }
  return array(0,$sequences);
}

function get_variable_sites ($sequences) {
	$maxlength =0;
	foreach ($sequences as $seq) {
		// find longest seq
		$maxlength = max($maxlength, strlen($seq[1]));
	}
	//
	// Identify variable sites
	//
	$variable_sites=array();
	for ($i = 0; $i <= $maxlength; $i++) {
	$refbase = $sequences[0][1][$i];
		for ($j=1; $j < count($sequences); $j++) {
			$curbase=$sequences[$j][1][$i];
			if ($refbase <> $curbase) {
				$variable_sites[]= $i; # Recode to 1st base = 1 etc.
				break;
				}
		}
	}
	return $variable_sites;
}

$colours=array();
	$colours["A"] = array("#00DD00","#FFFFFF"); # GREEN
	$colours["C"] = array("#0000DD","#FFFFFF"); # BLUE
	$colours["G"] = array("#000000","#FFFFFF"); # BLACK
	$colours["T"] = array("#DD0000","#FFFFFF"); # RED

function dna_colour($b, $state=False) {
global $colours;
if ($state) {
	return "<span style='background-color : ".$colours[$b][0]."; color : ".$colours[$b][1]."; '>".$b."</span>";
	} 
return $b;
}

function format_report ($sequences, $variable_sites, $zerobase=False) {
	$zerobase_hack = 1;
	if ($zerobase) {
		$zerobase_hack = 0;
	}
	// Create report
	$out="";
	$out2="";

	//
	// Find the longest sequence
	//
	$max_header_length= 0;
	foreach ($sequences as $seq) { 
		$max_header_length = max($max_header_length, strlen($seq[0]));
		} 
	//
	// Initialize position legend lines with spaces
	//
	$pos_legend_lines = max(1, intval(log10( max($variable_sites)))+1 ); # Calculate number of lines, i.e. three lines for hundreds of bases, four lines for thousands of bases etc.
	$pos_legend = array(); # Will make the position legend as a big table of text.
	for ($i = 0; $i < $pos_legend_lines; $i++) { # I.e. for all three lines make them the length of the longests sequence and fill them up with spaces.
		$pos_legend[$i] = str_pad("",$max_header_length, " ", STR_PAD_LEFT)." ";
	}
	//
	// Print numbers in vertical columns
	//
	foreach ($variable_sites as $position) {
		for ($i = 0; $i < $pos_legend_lines; $i++) {
			$pos_legend[$i] .= substr( str_pad($position + $zerobase_hack, $pos_legend_lines, " ", STR_PAD_LEFT), $i, 1) ;
		}
	}
	//
	// End position legend
	//
	if ($_POST['showPOS']) {
		foreach ($pos_legend as $line) { 
			$out .= $line."\n"; 
			}
		}
	//
	// Get sequence line of variable bases 
	//
	for ($j=0; $j < count($sequences); $j++) {
		$seq = $sequences[$j];
		$header = $seq[0];
		$bases = strtoupper($seq[1]);
		$out .= str_pad($header,$max_header_length, " ", STR_PAD_LEFT)." ";
		$out2 .= ">".$header."\n";
		foreach ($variable_sites as $index) {
			if ($_POST['showDOT']) {
				$ref_base=$sequences[0][1][$index];
				if (($j > 0) and ($ref_base == $bases[$index]) ) {
					$out .= ".";
					$out2 .= ".";
				} else {
				$out .= dna_colour($bases[$index],$_POST['colorDNA']);
				$out2 .= $bases[$index];
				}
			} 
			else {
			$out .= dna_colour($bases[$index],$_POST['colorDNA']);
			$out2 .= $bases[$index];
			}
		}
		$out .= "<br>";
		$out2 .= "\n";
	}
	//
	// Formatting of the two outputs. HTML and text output
	//
	$r['text'].="";
	$r['text'].=$out2;
	$r['text'].="";

	$r['html'].="<PRE>";
	$r['html'].=$out;
	$r['html'].="</PRE>";
	
	$r['textlabel']="Alignment of variable sites in fasta format";
	$r['htmllabel']="Alignment of variable sites only";
	$r['filename']="variable_sites.fasta";
	$r['filelabel']="Save reduced alignment to disk ";
	return $r;
}

if ($submit) {
  $r= parse_sequences($inputseqs);
  if ($r[0]=="0") {
    $sequences=$r[1];
	$variable_sites =get_variable_sites($sequences); 
    $al= format_report($sequences, $variable_sites, $_POST['zerobase']);
  } else {
    $error = $r[1];
  }
  unset($r);
}
//
// BEGIN HTML OUTPUT!
//
show_page_header("Extract variable sites from alignment 1.1", "<a href='#formtools.results'>Analysis finished, results are at bottom (or click here)</a>", $submit, $error);
?>
<!-- INPUT FORM -->
<form action="#" method='POST' enctype='multipart/form-data'>
<?php
show_fieldset_inputseqs ($submit_example, $inputseqs, "./examples/alignment_variable_sites.fa") 
?>
<P>
<!-- Settings fieldset -->
<?php
if (!$submit or $submit_example) {
$_POST['colorDNA'] = True;
$_POST['showPOS'] = True;
}

?>
<fieldset>
<legend>Settings</legend>
<P>
<label>Color DNA bases</label>
<input type="checkbox" name ="colorDNA" <?php if ($_POST['colorDNA']) { echo 'CHECKED'; } ?> >
</p>
<P>
<label>Show alignment position above each variable site</label>
<input type="checkbox" name ="showPOS" <?php if ($_POST['showPOS']) { echo 'CHECKED'; } ?> >
</p>
<br>
<P>
<label>Show identical bases as a dot</label>
<input type="checkbox" name ="showDOT" <?php if ($_POST['showDOT']) { echo 'CHECKED'; } ?> >
</p>
<P>
<label>0-based position numbering?<br>(1st base = 0 etc.)</label>
<input type="checkbox" name ="zerobase" <?php if ($_POST['zerobase']) { echo 'CHECKED'; } ?> >
</p>

<?php
show_submit_buttons("Extract variable sites");
?>
</fieldset>
</form>

<?php
if ($submit and !$error) {
  $b[] = get_save_button($al, 'r1');
  print get_save_form($b, "Save cropped alignment");
  print get_results_from_buttons($b);
}

show_page_footer();
?>
