<?php
require('./tools/seqtools.php');// include sequence functions, parsing etc.
require('./tools/formtools.php'); // get form tools

$debug=0;

#
# Form buttons from main form
#
$submit         = get_post_value('submit');
$submit_example = get_post_value('submit_example');
$gapchar        = get_post_value('gapchar');
$prefix         = get_post_value('prefix');
$firstheader    = get_post_value('firstheader');
$adv_delimiter  = get_post_value('adv_delimiter');

#
# Analysis
#

function parse_sequences($sequences) {
  $k = 0;// initalise haplotype number

  $gapchar       = get_post_value('gapchar');
  $prefix        = get_post_value('prefix');
  $firstheader   = get_post_value('firstheader');
  $adv_delimiter = get_post_value('adv_delimiter');
  #
  $haplotypes = array();
  for ($i=0; $i<count($sequences); $i++) {// all sequences
    $hit = 0;
	# Check all existing haplotypes for match to current sequence
    for ($j=0; $j<=$k; $j++){
		if (!isset($haplotypes[$j])) { 
			break;
		}
		if ($sequences[$i][1] == $haplotypes[$j][1]){
			# add header of sequence $i to list of sequence headers with haplotype $j
			$haplotypes[$j][0][] = $sequences[$i][0];
			$haplotype = $j; // Put haplotype number to sequence
			$hit = 1;// found existing haplotype
			break ;
		}
    }
    // All haplotypes searched
    // Check to see if anything was found
    // if not, create new haplotype
    if ($hit == 0) {
      # add header of sequence $i to list of sequences with haplotype $k
	  $haplotypes[$k][0][] = $sequences[$i][0];
	  # Set haplotype DNA sequence
	  $haplotypes[$k][1] = $sequences[$i][1];
      # Is the keep sequence headers flag set?
	  if ($firstheader) {
		$haplotypes[$k][2] = $sequences[$i][0];
      } else {
		$haplotypes[$k][2] = $prefix.($k+1); 
      }
      $haplotypes[$k][3] = $k+1; // number
      $haplotype         =$k;
      $hit = 0;
      $k++;
    }
	
    // Done for all sequences
    $h   = explode($adv_delimiter, $sequences[$i][0]);
    $id  = $h[0];
    $sex = $h[1];
    $pop = $h[2];

    $sequences[$i]['id']        = $id;
    $sequences[$i]['pop']       = $pop;
    $sequences[$i]['sex']       = $sex; 
    $sequences[$i]['haplotype'] = $haplotype;
  }
  $r = array();
  $r['haplotypes'] = $haplotypes;
  $r['sequences']  = $sequences;
  return ($r);
}


function get_sexdata ($sequences) {
  $sexdata = array();
  for ($i=0; $i<count($sequences); $i++) {// all sequences
    $seq = $sequences[$i];
    $id  = $seq['id'];
    $pop = $seq['pop'];
    $sex = $seq['sex'];
    $haplotype = $seq['haplotype'];
	if (!isset($sexdata[$sex])) { 
		$sexdata[$sex] = array();
	}
	if (!isset($sexdata[$sex]['haplotypes'][$haplotype])) { 
		$sexdata[$sex]['haplotypes'][$haplotype] = 0;
	}
    $sexdata[$sex]['haplotypes'][$haplotype] += 1;
    $sexdata[$sex]['sequences'][] = $i; // putting sequence number in sex group
  }
  ksort($sexdata);
  return $sexdata;
}

function get_popdata ($sequences, $sexfilter="") {
  for ($i=0; $i<count($sequences); $i++) {// all sequences
    $seq = $sequences[$i];
    $id  = $seq['id'];
    $pop = $seq['pop'];
    $sex = $seq['sex'];
    $haplotype = $seq['haplotype'];
    if ($sex==$sexfilter or $sexfilter=="") {
		if (!isset($popdata[$pop]['haplotypes'][$haplotype])) { 
			$popdata[$pop]['haplotypes'][$haplotype] = 0;
		}
		$popdata[$pop]['haplotypes'][$haplotype] += 1;
		$popdata[$pop]['sequences'][]= $i; // putting sequence number in pop group
    }
  }
  ksort($popdata);
  return $popdata;
}

function frequency_report ($haplotypes) {
  // create report1
  $report_freq = "<table class='tableborder'>";
  $report_freq .= "<tr><td>Haplotype number</td><td>Label</td><td>Number of sequences</td><td>Length of haplotype (bp)</td><td>Sequences belonging to haplotype</td>";
  $report_freq .= "</tr>";
  
  foreach ($haplotypes as $current){// put each haplotype in $current
    $report_freq .="<tr><td valign='top'>$current[3]</td>";// Haplotype number
    $report_freq .="<td valign='top'>$current[2]</td>";// Haplotype label
    $report_freq .="<td valign='top'>".count($current[0])."</td>";//counting sequences
    $report_freq .="<td valign='top'>".strlen($current[1])."</td>";
    $report_freq .="<td>";
    foreach ($current[0] as $sequenceheader){
      $report_freq .= $sequenceheader."<br>";// Sequence belonging to haplotype
    }
    $report_freq .= "</td></tr>";
  }
  $report_freq .= "</table>";

  $r['text'] = $report_freq;
  $r['html'] = $report_freq;

  $r['textlabel']="Frequency report in html format";
  $r['htmllabel']="Frequencies and report of haplotypes";

  $r['filename']="frequency_report.html";
  $r['filelabel']="Frequency report in html format";
  return $r;

}


function sex_report($sexdata, $haplotypes) {
  $sexreport="<table class='tableborder'><tr><td>Sex</td><td>Number of individuals</td><td>Number of haplotypes</td><td>Haplotype</td><td>Label</td><td>Frequency</td><tr>";
  foreach($sexdata AS $sex => $r){
    $ht_list=$r['haplotypes'];
    $rs=count($ht_list);
    $sexreport .= "<tr><td rowspan=$rs>$sex</td><td rowspan=$rs>".array_sum($ht_list)."</td><td rowspan=$rs>".count($ht_list)."</td>";
    ksort($ht_list);
    foreach($ht_list AS $ht => $ids){
      $sexreport.="<td>".$haplotypes[$ht][3]."</td><td>".$haplotypes[$ht][2]."</td><td>$ids</td></tr><tr>";
    }
    $sexreport .= "</tr>"; 
  }
  $sexreport.="</table>";

  $r['text'] = $sexreport;
  $r['html'] = $sexreport;

  $r['textlabel']="Sex specific report in html format";
  $r['htmllabel']="Sex specific report in html format";

  $r['filename']="sex_report.html";
  $r['filelabel']="Sex specific report in html format";
  return $r;


}

function sequences2fasta_local($haplotypes) {
  $fullset ="";
  foreach ($haplotypes as $current){
    $fullset .= ">".$current[2]."\n";
    $fullset .= wrap_sequence($current[1], 70, "\n");
    $fullset .= "\n";
  }
  
  $r['text'] = $fullset;
  $r['html'] = "<PRE>".$fullset."</PRE>\n";

  $r['textlabel']="Haplotypes in fasta format";
  $r['htmllabel']="Haplotypes in fasta format";

  $r['filename']="all_haplotype.fasta";
  $r['filelabel']="Haplotypes in fasta format";
  return $r;
}

function population_report($popdata,$haplotypes) {
  $popreport="<table class='tableborder'><tr><td>Population</td><td>Number of individuals</td><td>Number of haplotypes</td><td>Haplotype</td><td>Label</td><td>Frequency</td><tr>";
  foreach($popdata AS $pop => $r ){
    $ht_list=$r['haplotypes'];
    $rs=count($ht_list);
    $popreport .= "<tr><td rowspan=$rs>$pop</td><td rowspan=$rs>".array_sum($ht_list)."</td><td rowspan=$rs>".count($ht_list)."</td>";
    ksort($ht_list);
    foreach($ht_list AS $ht => $ids){
      $popreport.="<td>".$haplotypes[$ht][3]."</td><td>".$haplotypes[$ht][2]."</td><td>$ids</td></tr><tr>";
    }
    $popreport .= "</tr>"; 
  }
  $popreport.="</table>";

  $r['text'] = $popreport;
  $r['html'] = $popreport;

  $r['textlabel']="Population specific report in html format";
  $r['htmllabel']="Population specific report in html format";

  $r['filename']="pop_report.html";
  $r['filelabel']="Population specific report in html format";

  return $r;
}

function arlequin_report ($popdata, $haplotypes, $sex="") {
  $pops=count($popdata); // number of populations
  $title="DNA collapser project file for arlequin";
  $name="A group of {$pops} populations analysed for DNA";
  if ($sex <> "") {
    $title .=" ($sex only)";
    $name .= " ($sex only)";
  } else {
    $title.=" (all data)";
    $name.=" (all data)";
  }
  $arlequin=<<<ARL
[Profile]
\t Title="$title"
\t NbSamples={$pops}
\t GenotypicData=0
\t DataType=DNA
\t LocusSeparator=NONE
[Data]
\t [[Samples]]\n
ARL;
#}

foreach($popdata AS $pop => $r ){
  $ht_list=$r['haplotypes'];
  $arlequin.= "\t\t SampleName=\"Population $pop\"\n";
  $arlequin.= "\t\t SampleSize=".array_sum($ht_list)." \n";
  $arlequin.= "\t\t SampleData= {\n";
  foreach ($ht_list AS $ht => $ids) {
    $arlequin.= "\t\t\t ".$haplotypes[$ht][3]." \t $ids \t ".$haplotypes[$ht][1]."\n";
   }
  $arlequin.= "\t\t}\n";
}

$arlequin.=<<<ARL
[[Structure]]
\t StructureName="$name"
\t NbGroups=1
\t Group=\t { \n
ARL;
	
#}
 foreach($popdata AS $pop =>$r ){
   $arlequin.= "\t\t \"Population $pop\"\n";
 }
 
 $arlequin.= "\t\t}\n";


  $r['text'] = $arlequin;
  $r['html'] = "<PRE>".$arlequin."</PRE>";

  $r['textlabel']="Arlequin project file";
  $r['htmllabel']="Arlequin project file";
  $r['filename']="arlequin_project_file.arp";
  $r['filelabel']="Arlequin project file";
  if ($sex<>"") {
    $r['textlabel'].=" for $sex";
    $r['htmllabel'].=" for $sex";
    $r['filename'] ="arlequin_project_file_$sex.arp";
    $r['filelabel'].=" for $sex";
  } else {
    $r['filelabel'].=" for all data";
  }
  return $r;
}

$error=null;

if ($submit) {
  $sequences = parse_fasta($inputseqs, 1, $gapchar); // returns array of sequences, each consist of two elements: header and sequence, i.e. $var[0][0] = header of seq 1, $var[0][1] = sequence 1 etc...
  if (count($sequences) ==0){
    $error="Couldn't parse FASTA input correctly, please submit fasta formatted sequences";
  }  
}

//
// BEGIN HTML OUTPUT!
//
show_page_header("DNAcollapser", "<a href='#formtools.results'>Analysis finished, results are at bottom (or click here)</a>", $submit, $error);
?>
<!-- INPUT FORM -->
<form action="#" method='POST' enctype='multipart/form-data'>
<?php
show_fieldset_inputseqs ($submit_example, $inputseqs, "./examples/dnacollapser.fa") 
?>
<P>
<fieldset>
<legend>Settings</legend>
<table>
<tr>
<td>
<p><label for="gapchar">Gap character (1)</label> <input type="text" name ="gapchar" size=10 value="<?php echo $gapchar;?>"/></p>
<P><label for="adv_limiter">Header field delimiter (2)</label><input type="text" name ="adv_delimiter" size=10 value="<?php if($adv_delimiter) {echo $adv_delimiter;} else {echo "|";} ?>"/></p>
</td>
<td>
<p><label for="prefix">Haplotype prefix OR</label> <input type="text" size=10 name ="prefix" value="<?php echo $prefix;?>"/></p>
<P><label for="firstheader">Keep sequence headers (3)</label><input type="checkbox" name="firstheader" value="firstheader" <?php if($firstheader){echo 'checked';}?>/></p>
</td></tr></table>
<?php
show_submit_buttons("Collapse sequences");
?>
<div id='small'>
(1) Optional - leave blank to keep gaps<br>
(2) Sequence header can contain information about ID, Sex, Population in the following format: <strong>>ID [delimiter] Sex [delimiter] Population</strong><BR>
(3) Create prefixed haplotype headers or use a representative sequence header for haplotype headers<br>
</div>
</fieldset>
</form>

<!-- RESULTS SECTION FOLLOW -->
<?php
if ($error) {
     print "<h1><font color ='#AA0000'><br> $error</font></h1>\n";
}

if ($submit and !$error) {
  $r=parse_sequences($sequences);
  $sequences=$r['sequences'];
  $haplotypes=$r['haplotypes'];
  
  $sexdata=get_sexdata($sequences);
  $popdata=get_popdata($sequences);
  
  // Reports
  
  $haplotypes_fasta=sequences2fasta_local($haplotypes);
  $frequency_report = frequency_report($haplotypes);
  $sex_report = sex_report($sexdata,$haplotypes);
  $pop_report = population_report($popdata, $haplotypes);

  print "<a name='formtools.results'>";
 
  print "<h3>".count($sequences)." sequences were collapsed into ".count($haplotypes)." haplotypes</h3>\n";
  //
  // Get buttons from all the resulting objects
  //
  $b[] = get_save_button($frequency_report, "r2");
  $b[] = get_save_button($pop_report, "r4");
  $b[] = get_save_button($sex_report, "r3");
  $b[] = get_save_button($haplotypes_fasta, "r1");

  $arl=arlequin_report($popdata,$haplotypes);
  $b[] = get_save_button($arl, "r5");

  if (count($sexdata) > 1) {
    foreach ($sexdata as $sex => $values){
      // create new popdata and get arlequin
      $pop2=get_popdata($sequences,$sex);
      $arl=arlequin_report($pop2, $haplotypes, $sex);
      $b[] = get_save_button($arl, "r5.$sex");
    }
  }
  
  print get_save_form($b, "Save results to disk");
  print "</form>";
  
  print get_results_from_buttons($b);
}
   
 

//
// END HTML OUTPUT
//

function show_debug ($s,$l) {
  print "<h1>$l</h1><PRE>";
  print_r($s);
  print "</PRE>";
  return 0;

}  

if ($debug ==1) {
  show_debug($sequences, "sequences");
  show_debug($haplotypes, "haplotypes");
  show_debug($popdata, "popdata");
  show_debug($sexdata, "sexdata");

}
?>

<?php
show_page_footer();
?>
